# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import lora_get_params, lora_list_params, lora_load_params, lora_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorLora, AsyncCursorLora
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.deployed_model import DeployedModel

__all__ = ["LoraResource", "AsyncLoraResource"]


class LoraResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LoraResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return LoraResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LoraResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return LoraResourceWithStreamingResponse(self)

    def update(
        self,
        deployed_model_id: str,
        *,
        account_id: str | None = None,
        default: bool | Omit = omit,
        deployment: str | Omit = omit,
        description: str | Omit = omit,
        display_name: str | Omit = omit,
        model: str | Omit = omit,
        public: bool | Omit = omit,
        serverless: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeployedModel:
        """
        Update LoRA

        Args:
          default: If true, this is the default target when querying this model without the
              `#<deployment>` suffix. The first deployment a model is deployed to will have
              this field set to true.

          deployment: The resource name of the base deployment the model is deployed to.

          description: Description of the resource.

          public: If true, the deployed model will be publicly reachable.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not deployed_model_id:
            raise ValueError(f"Expected a non-empty value for `deployed_model_id` but received {deployed_model_id!r}")
        return self._patch(
            f"/v1/accounts/{account_id}/deployedModels/{deployed_model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/deployedModels/{deployed_model_id}",
            body=maybe_transform(
                {
                    "default": default,
                    "deployment": deployment,
                    "description": description,
                    "display_name": display_name,
                    "model": model,
                    "public": public,
                    "serverless": serverless,
                },
                lora_update_params.LoraUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeployedModel,
        )

    def list(
        self,
        *,
        account_id: str | None = None,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorLora[DeployedModel]:
        """
        List LoRAs

        Args:
          filter: Only depoyed models satisfying the provided filter (if specified) will be
              returned. See https://google.aip.dev/160 for the filter grammar.

          order_by: A comma-separated list of fields to order by. e.g. "foo,bar" The default sort
              order is ascending. To specify a descending order for a field, append a " desc"
              suffix. e.g. "foo desc,bar" Subfields are specified with a "." character. e.g.
              "foo.bar" If not specified, the default order is by "name".

          page_size: The maximum number of deployed models to return. The maximum page_size is 200,
              values above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListDeployedModels call. Provide this to
              retrieve the subsequent page. When paginating, all other parameters provided to
              ListDeployedModels must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            f"/v1/accounts/{account_id}/deployedModels"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/deployedModels",
            page=SyncCursorLora[DeployedModel],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    lora_list_params.LoraListParams,
                ),
            ),
            model=DeployedModel,
        )

    def get(
        self,
        deployed_model_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeployedModel:
        """Get LoRA

        Args:
          read_mask: The fields to be returned in the response.

        If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not deployed_model_id:
            raise ValueError(f"Expected a non-empty value for `deployed_model_id` but received {deployed_model_id!r}")
        return self._get(
            f"/v1/accounts/{account_id}/deployedModels/{deployed_model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/deployedModels/{deployed_model_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"read_mask": read_mask}, lora_get_params.LoraGetParams),
            ),
            cast_to=DeployedModel,
        )

    def load(
        self,
        *,
        account_id: str | None = None,
        replace_merged_addon: bool | Omit = omit,
        default: bool | Omit = omit,
        deployment: str | Omit = omit,
        description: str | Omit = omit,
        display_name: str | Omit = omit,
        model: str | Omit = omit,
        public: bool | Omit = omit,
        serverless: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeployedModel:
        """
        Load LoRA

        Args:
          replace_merged_addon: Merges new addon to the base model, while unmerging/deleting any existing addon
              in the deployment. Must be specified for hot reload deployments

          default: If true, this is the default target when querying this model without the
              `#<deployment>` suffix. The first deployment a model is deployed to will have
              this field set to true.

          deployment: The resource name of the base deployment the model is deployed to.

          description: Description of the resource.

          public: If true, the deployed model will be publicly reachable.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            f"/v1/accounts/{account_id}/deployedModels"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/deployedModels",
            body=maybe_transform(
                {
                    "default": default,
                    "deployment": deployment,
                    "description": description,
                    "display_name": display_name,
                    "model": model,
                    "public": public,
                    "serverless": serverless,
                },
                lora_load_params.LoraLoadParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"replace_merged_addon": replace_merged_addon}, lora_load_params.LoraLoadParams),
            ),
            cast_to=DeployedModel,
        )

    def unload(
        self,
        deployed_model_id: str,
        *,
        account_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Unload LoRA

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not deployed_model_id:
            raise ValueError(f"Expected a non-empty value for `deployed_model_id` but received {deployed_model_id!r}")
        return self._delete(
            f"/v1/accounts/{account_id}/deployedModels/{deployed_model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/deployedModels/{deployed_model_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncLoraResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLoraResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncLoraResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLoraResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return AsyncLoraResourceWithStreamingResponse(self)

    async def update(
        self,
        deployed_model_id: str,
        *,
        account_id: str | None = None,
        default: bool | Omit = omit,
        deployment: str | Omit = omit,
        description: str | Omit = omit,
        display_name: str | Omit = omit,
        model: str | Omit = omit,
        public: bool | Omit = omit,
        serverless: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeployedModel:
        """
        Update LoRA

        Args:
          default: If true, this is the default target when querying this model without the
              `#<deployment>` suffix. The first deployment a model is deployed to will have
              this field set to true.

          deployment: The resource name of the base deployment the model is deployed to.

          description: Description of the resource.

          public: If true, the deployed model will be publicly reachable.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not deployed_model_id:
            raise ValueError(f"Expected a non-empty value for `deployed_model_id` but received {deployed_model_id!r}")
        return await self._patch(
            f"/v1/accounts/{account_id}/deployedModels/{deployed_model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/deployedModels/{deployed_model_id}",
            body=await async_maybe_transform(
                {
                    "default": default,
                    "deployment": deployment,
                    "description": description,
                    "display_name": display_name,
                    "model": model,
                    "public": public,
                    "serverless": serverless,
                },
                lora_update_params.LoraUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeployedModel,
        )

    def list(
        self,
        *,
        account_id: str | None = None,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DeployedModel, AsyncCursorLora[DeployedModel]]:
        """
        List LoRAs

        Args:
          filter: Only depoyed models satisfying the provided filter (if specified) will be
              returned. See https://google.aip.dev/160 for the filter grammar.

          order_by: A comma-separated list of fields to order by. e.g. "foo,bar" The default sort
              order is ascending. To specify a descending order for a field, append a " desc"
              suffix. e.g. "foo desc,bar" Subfields are specified with a "." character. e.g.
              "foo.bar" If not specified, the default order is by "name".

          page_size: The maximum number of deployed models to return. The maximum page_size is 200,
              values above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListDeployedModels call. Provide this to
              retrieve the subsequent page. When paginating, all other parameters provided to
              ListDeployedModels must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            f"/v1/accounts/{account_id}/deployedModels"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/deployedModels",
            page=AsyncCursorLora[DeployedModel],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    lora_list_params.LoraListParams,
                ),
            ),
            model=DeployedModel,
        )

    async def get(
        self,
        deployed_model_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeployedModel:
        """Get LoRA

        Args:
          read_mask: The fields to be returned in the response.

        If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not deployed_model_id:
            raise ValueError(f"Expected a non-empty value for `deployed_model_id` but received {deployed_model_id!r}")
        return await self._get(
            f"/v1/accounts/{account_id}/deployedModels/{deployed_model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/deployedModels/{deployed_model_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"read_mask": read_mask}, lora_get_params.LoraGetParams),
            ),
            cast_to=DeployedModel,
        )

    async def load(
        self,
        *,
        account_id: str | None = None,
        replace_merged_addon: bool | Omit = omit,
        default: bool | Omit = omit,
        deployment: str | Omit = omit,
        description: str | Omit = omit,
        display_name: str | Omit = omit,
        model: str | Omit = omit,
        public: bool | Omit = omit,
        serverless: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeployedModel:
        """
        Load LoRA

        Args:
          replace_merged_addon: Merges new addon to the base model, while unmerging/deleting any existing addon
              in the deployment. Must be specified for hot reload deployments

          default: If true, this is the default target when querying this model without the
              `#<deployment>` suffix. The first deployment a model is deployed to will have
              this field set to true.

          deployment: The resource name of the base deployment the model is deployed to.

          description: Description of the resource.

          public: If true, the deployed model will be publicly reachable.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            f"/v1/accounts/{account_id}/deployedModels"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/deployedModels",
            body=await async_maybe_transform(
                {
                    "default": default,
                    "deployment": deployment,
                    "description": description,
                    "display_name": display_name,
                    "model": model,
                    "public": public,
                    "serverless": serverless,
                },
                lora_load_params.LoraLoadParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"replace_merged_addon": replace_merged_addon}, lora_load_params.LoraLoadParams
                ),
            ),
            cast_to=DeployedModel,
        )

    async def unload(
        self,
        deployed_model_id: str,
        *,
        account_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Unload LoRA

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not deployed_model_id:
            raise ValueError(f"Expected a non-empty value for `deployed_model_id` but received {deployed_model_id!r}")
        return await self._delete(
            f"/v1/accounts/{account_id}/deployedModels/{deployed_model_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/deployedModels/{deployed_model_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class LoraResourceWithRawResponse:
    def __init__(self, lora: LoraResource) -> None:
        self._lora = lora

        self.update = to_raw_response_wrapper(
            lora.update,
        )
        self.list = to_raw_response_wrapper(
            lora.list,
        )
        self.get = to_raw_response_wrapper(
            lora.get,
        )
        self.load = to_raw_response_wrapper(
            lora.load,
        )
        self.unload = to_raw_response_wrapper(
            lora.unload,
        )


class AsyncLoraResourceWithRawResponse:
    def __init__(self, lora: AsyncLoraResource) -> None:
        self._lora = lora

        self.update = async_to_raw_response_wrapper(
            lora.update,
        )
        self.list = async_to_raw_response_wrapper(
            lora.list,
        )
        self.get = async_to_raw_response_wrapper(
            lora.get,
        )
        self.load = async_to_raw_response_wrapper(
            lora.load,
        )
        self.unload = async_to_raw_response_wrapper(
            lora.unload,
        )


class LoraResourceWithStreamingResponse:
    def __init__(self, lora: LoraResource) -> None:
        self._lora = lora

        self.update = to_streamed_response_wrapper(
            lora.update,
        )
        self.list = to_streamed_response_wrapper(
            lora.list,
        )
        self.get = to_streamed_response_wrapper(
            lora.get,
        )
        self.load = to_streamed_response_wrapper(
            lora.load,
        )
        self.unload = to_streamed_response_wrapper(
            lora.unload,
        )


class AsyncLoraResourceWithStreamingResponse:
    def __init__(self, lora: AsyncLoraResource) -> None:
        self._lora = lora

        self.update = async_to_streamed_response_wrapper(
            lora.update,
        )
        self.list = async_to_streamed_response_wrapper(
            lora.list,
        )
        self.get = async_to_streamed_response_wrapper(
            lora.get,
        )
        self.load = async_to_streamed_response_wrapper(
            lora.load,
        )
        self.unload = async_to_streamed_response_wrapper(
            lora.unload,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional

from ..._models import BaseModel
from .new_log_probs import NewLogProbs

__all__ = ["RawOutput"]


class RawOutput(BaseModel):
    """
    Extension of OpenAI that returns low-level interaction of what the model
    sees, including the formatted prompt and function calls
    """

    completion: str
    """Raw completion produced by the model before any tool calls are parsed"""

    prompt_fragments: List[Union[str, int]]
    """
    Pieces of the prompt (like individual messages) before truncation and
    concatenation. Depending on prompt_truncate_len some of the messages might be
    dropped. Contains a mix of strings to be tokenized and individual tokens (if
    dictated by the conversation template)
    """

    prompt_token_ids: List[int]
    """Fully processed prompt as seen by the model"""

    completion_logprobs: Optional[NewLogProbs] = None
    """OpenAI-compatible log probabilities format"""

    completion_token_ids: Optional[List[int]] = None
    """Token IDs for the raw completion"""

    grammar: Optional[str] = None
    """
    Grammar used for constrained decoding, can be either user provided (directly or
    JSON schema) or inferred by the chat template
    """

    images: Optional[List[str]] = None
    """Images in the prompt"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TransformedParam"]


class TransformedParam(TypedDict, total=False):
    source_dataset_id: Required[Annotated[str, PropertyInfo(alias="sourceDatasetId")]]

    filter: str

    original_format: Annotated[
        Literal["FORMAT_UNSPECIFIED", "CHAT", "COMPLETION", "RL"], PropertyInfo(alias="originalFormat")
    ]



import os
import json
import time

import httpx
from dotenv import load_dotenv  # type: ignore[import-not-found]

from fireworks import Fireworks, ConflictError
from fireworks.types import Dataset

load_dotenv()

# Constants
TRAIN_FILE_NAME = "train.jsonl"
EVALUATION_FILE_NAME = "evaluation.jsonl"
TRAIN_DATASET_ID = "train-dataset"
EVALUATION_DATASET_ID = "evaluation-dataset"
MODEL_NAME = "accounts/fireworks/models/qwen3-30b-a3b-instruct-2507"
SFTJ_DISPLAY_NAME = "My Supervised Fine-Tuning Job"
SFTJ_LEARNING_RATE = 1e-5
SFTJ_LEARNING_RATE_WARMUP_STEPS = 200
SFTJ_EPOCHS = 1
SFTJ_LORA_RANK = 16
SFTJ_GRADIENT_ACCUMULATION_STEPS = 2
SFTJ_BATCH_SIZE = 16384
SFTJ_MAX_CONTEXT_LENGTH = 16384


def create_and_upload_dataset(client: Fireworks, dataset_id: str, file_name: str, example_count: int) -> Dataset:
    """Create a dataset, upload a file, and validate the upload to Fireworks."""
    # Create the dataset (skip if it already exists)
    try:
        dataset = client.datasets.create(
            dataset_id=dataset_id,
            dataset={
                "example_count": str(example_count),
            },
        )
    except ConflictError:
        print(f"Dataset {dataset_id} already exists, skipping creation")
        dataset = client.datasets.get(dataset_id=dataset_id)
        return dataset

    # Upload the file
    file_size = os.path.getsize(file_name)
    upload_endpoint = client.datasets.get_upload_endpoint(
        dataset_id=dataset_id,
        filename_to_size={
            file_name: str(file_size),
        },
    )

    if upload_endpoint.filename_to_signed_urls is None:
        raise ValueError("Failed to get upload endpoint URLs")

    signed_url = upload_endpoint.filename_to_signed_urls.get(file_name)
    if signed_url is None:
        raise ValueError(f"Failed to get signed URL for file: {file_name}")

    with open(file_name, "rb") as f:
        response = httpx.put(
            signed_url,
            content=f.read(),
            headers={
                "Content-Type": "application/octet-stream",
                "x-goog-content-length-range": f"{file_size},{file_size}",
            },
        )
        response.raise_for_status()

    # Validate the upload
    client.datasets.validate_upload(dataset_id=dataset_id, body={})

    return dataset


# 0) write jsonl to a file
train_data = [
    {
        "messages": [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "What is the capital of France?"},
            {"role": "assistant", "content": "Paris."},
        ]
    }
]
evaluation_data = [
    {
        "messages": [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "What is the capital of France?"},
            {"role": "assistant", "content": "Paris."},
        ]
    }
]
with open(TRAIN_FILE_NAME, "w") as f:
    for item in train_data:
        f.write(json.dumps(item) + "\n")
with open(EVALUATION_FILE_NAME, "w") as f:
    for item in evaluation_data:
        f.write(json.dumps(item) + "\n")

# Remember to set the environment variables:
# FIREWORKS_ACCOUNT_ID="your-account-id"
# FIREWORKS_API_KEY="your-api-key"
client = Fireworks()

# 1) Upload dataset
train_dataset = create_and_upload_dataset(client, TRAIN_DATASET_ID, TRAIN_FILE_NAME, len(train_data))
evaluation_dataset = create_and_upload_dataset(
    client, EVALUATION_DATASET_ID, EVALUATION_FILE_NAME, len(evaluation_data)
)

# 2) Create SFTJ
if train_dataset.name is None:
    raise ValueError("Train dataset name is None")
if evaluation_dataset.name is None:
    raise ValueError("Evaluation dataset name is None")

sftj = client.supervised_fine_tuning_jobs.create(
    dataset=train_dataset.name,
    evaluation_dataset=evaluation_dataset.name,
    base_model=MODEL_NAME,
    display_name=SFTJ_DISPLAY_NAME,
    learning_rate=SFTJ_LEARNING_RATE,
    learning_rate_warmup_steps=SFTJ_LEARNING_RATE_WARMUP_STEPS,
    epochs=SFTJ_EPOCHS,
    lora_rank=SFTJ_LORA_RANK,
    gradient_accumulation_steps=SFTJ_GRADIENT_ACCUMULATION_STEPS,
    batch_size=SFTJ_BATCH_SIZE,
    max_context_length=SFTJ_MAX_CONTEXT_LENGTH,
)

if sftj.name is None:
    raise ValueError("SFTJ name is None")

print("Go to the following URL to monitor the SFTJ:")
print(f"https://app.fireworks.ai/dashboard/fine-tuning/supervised/{sftj.name.split('/')[-1]}")

while sftj.state != "JOB_STATE_COMPLETED":
    time.sleep(5)
    if sftj.name is None:
        raise ValueError("SFTJ name is None")
    sftj_id = sftj.name.split("/")[-1]
    sftj = client.supervised_fine_tuning_jobs.get(sftj_id)
    print(f"SFTJ state: {sftj.state}")

print("SFTJ completed")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["DeploymentShapeVersionListResponse", "Snapshot"]


class Snapshot(BaseModel):
    """Full snapshot of the Deployment Shape at this version."""

    base_model: str = FieldInfo(alias="baseModel")

    accelerator_count: Optional[int] = FieldInfo(alias="acceleratorCount", default=None)
    """
    The number of accelerators used per replica. If not specified, the default is
    the estimated minimum required by the base model.
    """

    accelerator_type: Optional[
        Literal[
            "ACCELERATOR_TYPE_UNSPECIFIED",
            "NVIDIA_A100_80GB",
            "NVIDIA_H100_80GB",
            "AMD_MI300X_192GB",
            "NVIDIA_A10G_24GB",
            "NVIDIA_A100_40GB",
            "NVIDIA_L4_24GB",
            "NVIDIA_H200_141GB",
            "NVIDIA_B200_180GB",
            "AMD_MI325X_256GB",
        ]
    ] = FieldInfo(alias="acceleratorType", default=None)
    """
    The type of accelerator to use. If not specified, the default is
    NVIDIA_A100_80GB.
    """

    create_time: Optional[datetime] = FieldInfo(alias="createTime", default=None)
    """The creation time of the deployment shape."""

    description: Optional[str] = None
    """The description of the deployment shape.

    Must be fewer than 1000 characters long.
    """

    disable_deployment_size_validation: Optional[bool] = FieldInfo(
        alias="disableDeploymentSizeValidation", default=None
    )
    """If true, the deployment size validation is disabled."""

    display_name: Optional[str] = FieldInfo(alias="displayName", default=None)
    """Human-readable display name of the deployment shape.

    e.g. "My Deployment Shape" Must be fewer than 64 characters long.
    """

    draft_model: Optional[str] = FieldInfo(alias="draftModel", default=None)
    """The draft model name for speculative decoding.

    e.g. accounts/fireworks/models/my-draft-model If empty, speculative decoding
    using a draft model is disabled. Default is the base model's
    default_draft_model. this behavior.
    """

    draft_token_count: Optional[int] = FieldInfo(alias="draftTokenCount", default=None)
    """
    The number of candidate tokens to generate per step for speculative decoding.
    Default is the base model's draft_token_count.
    """

    enable_addons: Optional[bool] = FieldInfo(alias="enableAddons", default=None)
    """If true, LORA addons are enabled for deployments created from this shape."""

    enable_session_affinity: Optional[bool] = FieldInfo(alias="enableSessionAffinity", default=None)
    """Whether to apply sticky routing based on `user` field."""

    api_model_type: Optional[str] = FieldInfo(alias="modelType", default=None)
    """The model type of the base model."""

    name: Optional[str] = None

    ngram_speculation_length: Optional[int] = FieldInfo(alias="ngramSpeculationLength", default=None)
    """The length of previous input sequence to be considered for N-gram speculation."""

    num_lora_device_cached: Optional[int] = FieldInfo(alias="numLoraDeviceCached", default=None)

    parameter_count: Optional[str] = FieldInfo(alias="parameterCount", default=None)
    """The parameter count of the base model ."""

    precision: Optional[
        Literal[
            "PRECISION_UNSPECIFIED",
            "FP16",
            "FP8",
            "FP8_MM",
            "FP8_AR",
            "FP8_MM_KV_ATTN",
            "FP8_KV",
            "FP8_MM_V2",
            "FP8_V2",
            "FP8_MM_KV_ATTN_V2",
            "NF4",
            "FP4",
            "BF16",
            "FP4_BLOCKSCALED_MM",
            "FP4_MX_MOE",
        ]
    ] = None
    """The precision with which the model should be served."""

    preset_type: Optional[Literal["PRESET_TYPE_UNSPECIFIED", "MINIMAL", "FAST", "THROUGHPUT", "FULL_PRECISION"]] = (
        FieldInfo(alias="presetType", default=None)
    )
    """Type of deployment shape for different deployment configurations."""

    update_time: Optional[datetime] = FieldInfo(alias="updateTime", default=None)
    """The update time for the deployment shape."""


class DeploymentShapeVersionListResponse(BaseModel):
    create_time: Optional[datetime] = FieldInfo(alias="createTime", default=None)
    """The creation time of the deployment shape version.

    Lists will be ordered by this field.
    """

    latest_validated: Optional[bool] = FieldInfo(alias="latestValidated", default=None)
    """
    If true, this version is the latest validated version. Only one version of the
    shape can be the latest validated version.
    """

    name: Optional[str] = None

    public: Optional[bool] = None
    """If true, this version will be publicly readable."""

    snapshot: Optional[Snapshot] = None
    """Full snapshot of the Deployment Shape at this version."""

    validated: Optional[bool] = None
    """If true, this version has been validated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .new_log_probs_content_top_log_probs import NewLogProbsContentTopLogProbs

__all__ = ["NewLogProbsContent"]


class NewLogProbsContent(BaseModel):
    token: str

    bytes: List[int]

    logprob: float

    sampling_logprob: Optional[float] = None

    text_offset: int

    token_id: int

    extra_logprobs: Optional[List[float]] = None

    extra_tokens: Optional[List[int]] = None

    last_activation: Optional[str] = None

    routing_matrix: Optional[str] = None

    top_logprobs: Optional[List[NewLogProbsContentTopLogProbs]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared_params.wandb_config import WandbConfig

__all__ = ["SupervisedFineTuningJobCreateParams", "HiddenStatesGenConfig"]


class SupervisedFineTuningJobCreateParams(TypedDict, total=False):
    account_id: str

    dataset: Required[str]
    """The name of the dataset used for training."""

    supervised_fine_tuning_job_id: Annotated[str, PropertyInfo(alias="supervisedFineTuningJobId")]
    """
    ID of the supervised fine-tuning job, a random UUID will be generated if not
    specified.
    """

    base_model: Annotated[str, PropertyInfo(alias="baseModel")]
    """
    The name of the base model to be fine-tuned Only one of 'base_model' or
    'warm_start_from' should be specified.
    """

    batch_size: Annotated[int, PropertyInfo(alias="batchSize")]

    display_name: Annotated[str, PropertyInfo(alias="displayName")]

    early_stop: Annotated[bool, PropertyInfo(alias="earlyStop")]
    """Whether to stop training early if the validation loss does not improve."""

    epochs: int
    """The number of epochs to train for."""

    eval_auto_carveout: Annotated[bool, PropertyInfo(alias="evalAutoCarveout")]
    """Whether to auto-carve the dataset for eval."""

    evaluation_dataset: Annotated[str, PropertyInfo(alias="evaluationDataset")]
    """The name of a separate dataset to use for evaluation."""

    gradient_accumulation_steps: Annotated[int, PropertyInfo(alias="gradientAccumulationSteps")]

    hidden_states_gen_config: Annotated[HiddenStatesGenConfig, PropertyInfo(alias="hiddenStatesGenConfig")]
    """Config for generating dataset with hidden states for training."""

    is_turbo: Annotated[bool, PropertyInfo(alias="isTurbo")]
    """Whether to run the fine-tuning job in turbo mode."""

    jinja_template: Annotated[str, PropertyInfo(alias="jinjaTemplate")]

    learning_rate: Annotated[float, PropertyInfo(alias="learningRate")]
    """The learning rate used for training."""

    learning_rate_warmup_steps: Annotated[int, PropertyInfo(alias="learningRateWarmupSteps")]

    lora_rank: Annotated[int, PropertyInfo(alias="loraRank")]
    """The rank of the LoRA layers."""

    max_context_length: Annotated[int, PropertyInfo(alias="maxContextLength")]
    """The maximum context length to use with the model."""

    metrics_file_signed_url: Annotated[str, PropertyInfo(alias="metricsFileSignedUrl")]

    mtp_enabled: Annotated[bool, PropertyInfo(alias="mtpEnabled")]

    mtp_freeze_base_model: Annotated[bool, PropertyInfo(alias="mtpFreezeBaseModel")]

    mtp_num_draft_tokens: Annotated[int, PropertyInfo(alias="mtpNumDraftTokens")]

    nodes: int
    """The number of nodes to use for the fine-tuning job."""

    output_model: Annotated[str, PropertyInfo(alias="outputModel")]
    """The model ID to be assigned to the resulting fine-tuned model.

    If not specified, the job ID will be used.
    """

    region: Literal[
        "REGION_UNSPECIFIED",
        "US_IOWA_1",
        "US_VIRGINIA_1",
        "US_ILLINOIS_1",
        "AP_TOKYO_1",
        "US_ARIZONA_1",
        "US_TEXAS_1",
        "US_ILLINOIS_2",
        "EU_FRANKFURT_1",
        "US_TEXAS_2",
        "EU_ICELAND_1",
        "EU_ICELAND_2",
        "US_WASHINGTON_1",
        "US_WASHINGTON_2",
        "US_WASHINGTON_3",
        "AP_TOKYO_2",
        "US_CALIFORNIA_1",
        "US_UTAH_1",
        "US_GEORGIA_1",
        "US_GEORGIA_2",
        "US_WASHINGTON_4",
        "US_GEORGIA_3",
        "NA_BRITISHCOLUMBIA_1",
    ]
    """The region where the fine-tuning job is located."""

    wandb_config: Annotated[WandbConfig, PropertyInfo(alias="wandbConfig")]
    """The Weights & Biases team/user account for logging training progress."""

    warm_start_from: Annotated[str, PropertyInfo(alias="warmStartFrom")]
    """
    The PEFT addon model in Fireworks format to be fine-tuned from Only one of
    'base_model' or 'warm_start_from' should be specified.
    """


class HiddenStatesGenConfig(TypedDict, total=False):
    """Config for generating dataset with hidden states for training."""

    api_key: Annotated[str, PropertyInfo(alias="apiKey")]

    deployed_model: Annotated[str, PropertyInfo(alias="deployedModel")]

    input_limit: Annotated[int, PropertyInfo(alias="inputLimit")]

    input_offset: Annotated[int, PropertyInfo(alias="inputOffset")]

    max_tokens: Annotated[int, PropertyInfo(alias="maxTokens")]

    max_workers: Annotated[int, PropertyInfo(alias="maxWorkers")]

    output_activations: Annotated[bool, PropertyInfo(alias="outputActivations")]

    regenerate_assistant: Annotated[bool, PropertyInfo(alias="regenerateAssistant")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Required, TypeAlias, TypedDict

from .chat_completion_message_tool_call_function import ChatCompletionMessageToolCallFunction

__all__ = ["ChatCompletionMessageToolCall", "Function"]

Function: TypeAlias = Union[ChatCompletionMessageToolCallFunction, str]


class ChatCompletionMessageToolCall(TypedDict, total=False):
    function: Required[Function]
    """The function that the model called."""

    id: Optional[str]
    """The ID of the tool call."""

    type: str
    """The type of the tool. Currently, only `function` is supported."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .new_log_probs_content import NewLogProbsContent

__all__ = ["NewLogProbs"]


class NewLogProbs(BaseModel):
    """OpenAI-compatible log probabilities format"""

    content: Optional[List[NewLogProbsContent]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CodeSnippets"]


class CodeSnippets(BaseModel):
    entry_file: Optional[str] = FieldInfo(alias="entryFile", default=None)

    entry_func: Optional[str] = FieldInfo(alias="entryFunc", default=None)

    file_contents: Optional[Dict[str, str]] = FieldInfo(alias="fileContents", default=None)

    language: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .shared_params.wandb_config import WandbConfig
from .shared_params.training_config import TrainingConfig

__all__ = ["ReinforcementFineTuningStepCreateParams"]


class ReinforcementFineTuningStepCreateParams(TypedDict, total=False):
    account_id: str

    rlor_trainer_job_id: Annotated[str, PropertyInfo(alias="rlorTrainerJobId")]
    """ID of the RLOR trainer job, a random UUID will be generated if not specified."""

    dataset: str
    """The name of the dataset used for training."""

    display_name: Annotated[str, PropertyInfo(alias="displayName")]

    eval_auto_carveout: Annotated[bool, PropertyInfo(alias="evalAutoCarveout")]
    """Whether to auto-carve the dataset for eval."""

    evaluation_dataset: Annotated[str, PropertyInfo(alias="evaluationDataset")]
    """The name of a separate dataset to use for evaluation."""

    keep_alive: Annotated[bool, PropertyInfo(alias="keepAlive")]

    reward_weights: Annotated[SequenceNotStr[str], PropertyInfo(alias="rewardWeights")]
    """
    A list of reward metrics to use for training in format of
    "<reward_name>=<weight>".
    """

    rollout_deployment_name: Annotated[str, PropertyInfo(alias="rolloutDeploymentName")]
    """Rollout deployment name associated with this RLOR trainer job. This is optional.

    If not set, trainer will not trigger weight sync to rollout engine.
    """

    training_config: Annotated[TrainingConfig, PropertyInfo(alias="trainingConfig")]
    """Common training configurations."""

    wandb_config: Annotated[WandbConfig, PropertyInfo(alias="wandbConfig")]
    """The Weights & Biases team/user account for logging training progress."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["RollupSettings"]


class RollupSettings(BaseModel):
    criteria_weights: Optional[Dict[str, float]] = FieldInfo(alias="criteriaWeights", default=None)

    python_code: Optional[str] = FieldInfo(alias="pythonCode", default=None)

    skip_rollup: Optional[bool] = FieldInfo(alias="skipRollup", default=None)

    success_threshold: Optional[float] = FieldInfo(alias="successThreshold", default=None)
